searchNodes=[{"doc":"eharbor is an Erlang library that allows for running a function in a controlled manner, with the ability to deduplicate function calls with the same arguments. It features configurable deduplication, ordered function calls, and buffering to prevent system overload. This library is useful for running expensive tasks, where deduplication of the same inputs can lead to significant performance gains. The config type is a map with the following keys: name (default: default ) The name of the harbor instance. This name is used to derive a coordinator name and a pobox name. backlog (default: 1000 ) How big should our pobox buffer be? This is effectively going to determine how much memory the pobox will be allowed to consume. The underlying pobox acts as our load shedding for overload scenarios. Don't set this parameter to an arbitrary value instead try to calculate how much memory will use when the pobox will be full to make sure that the system will survive. piers (default: 100 ) How many unique parameters to process at once. This effectively limits concurrency on the backend. It can be used as a bulkhead by providing a limit that is smaller than connection pool size for a database like Postgres for example. breakwater_limit (default: 2000 ) Maximum number of processes waiting on a result being processed by the backend. This does not include the backlog processes. Keep in mind this value should not be to large otherwise memory spikes could occur during fanout of the results from conductor to follower. dedup (default: true ) Is this instance being used as a simple bulkhead or are we tring to accelerate the requests by deduplicating them. ordered (default: true ) For a sequential process P where no other sequential process writes. If the process reads and then writes when this parameter is set to false the write may appear to have occured before the read. This results in lower latency and lower overhead at the cost of ordering. The parameter is set to true by default since it that behavior is more intuitive. But if maximum performance is desired set it to false. group_by_key_fun (default: fun(Key) -&gt; Key end ) Sometimes some parameters need to be passed into harbor but shouldn't be considered for pier assignment or deduplication. This allows you to ignore some parameters while still passing them into your function. error_type (default: value ) How to handle scenarios where the pobox is full what type of error to return (should be one of these throw | error | raise | exit | value ). error_value (default: {error, full} ) The value of the throw error, exit etc. In this case we return a value because the default is value . buffer_insert_timeout (default: 5000 ) How long to wait for insertion into the pobox buffer normally changing this should not be required. follower_wait_for_conductor_timeout (default: infinity ) How long the follower should wait after the conductor. Normally you shouldn't have to modify this value since crashes are detected and will not cause followers to wait until timeout. conductor_wait_for_coordinator_followers_timeout (default: infinity ) Don't touch unless you know exactly what you are doing. assign_role_timeout (default: infinity ) Don't touch unless you know exactly what you are doing.","ref":"eharbor.html","title":"eharbor","type":"module"},{"doc":"Return the configuration defaults for harbor. config_defaults/2 returns the default configuration parameters. This is later used to convert an incomplete_config() into a config() . See also: merge_with_defaults/1 .","ref":"eharbor.html#config_defaults/0","title":"eharbor.config_defaults/0","type":"function"},{"doc":"From the harbor name return the derived coordinator name. The coordinator is a named process and to talk to it directly some operations require its name.","ref":"eharbor.html#coordinator_name/1","title":"eharbor.coordinator_name/1","type":"function"},{"doc":"Merge an incomplete configuration with the defaults to produce a proper config. merge_with_defaults/2 returns overridden parameters merged with the default configuration parameters when the paramter is missing. See also: config_defaults/0 .","ref":"eharbor.html#merge_with_defaults/1","title":"eharbor.merge_with_defaults/1","type":"function"},{"doc":"From the harbor name return the derived pobox name. The pobox is a named process and to talk to it directly some operations require its name.","ref":"eharbor.html#pobox_name/1","title":"eharbor.pobox_name/1","type":"function"},{"doc":"Run a function using an MFA with the config using a harbor instance.","ref":"eharbor.html#run/2","title":"eharbor.run/2","type":"function"},{"doc":"Run a function using a function reference and arguments with the config using a harbor instance.","ref":"eharbor.html#run/3","title":"eharbor.run/3","type":"function"},{"doc":"","ref":"eharbor.html#t:config/0","title":"eharbor.config/0","type":"type"},{"doc":"","ref":"eharbor.html#t:incomplete_config/0","title":"eharbor.incomplete_config/0","type":"type"},{"doc":"eharbor public API","ref":"eharbor_app.html","title":"eharbor_app","type":"module"},{"doc":"","ref":"eharbor_app.html#start/2","title":"eharbor_app.start/2","type":"function"},{"doc":"","ref":"eharbor_app.html#stop/1","title":"eharbor_app.stop/1","type":"function"},{"doc":"","ref":"eharbor_coordinator.html","title":"eharbor_coordinator","type":"module"},{"doc":"","ref":"eharbor_coordinator.html#get_stats/1","title":"eharbor_coordinator.get_stats/1","type":"function"},{"doc":"","ref":"eharbor_coordinator.html#handle_call/3","title":"eharbor_coordinator.handle_call/3","type":"function"},{"doc":"","ref":"eharbor_coordinator.html#handle_cast/2","title":"eharbor_coordinator.handle_cast/2","type":"function"},{"doc":"","ref":"eharbor_coordinator.html#handle_info/2","title":"eharbor_coordinator.handle_info/2","type":"function"},{"doc":"","ref":"eharbor_coordinator.html#init/1","title":"eharbor_coordinator.init/1","type":"function"},{"doc":"","ref":"eharbor_coordinator.html#pause/1","title":"eharbor_coordinator.pause/1","type":"function"},{"doc":"","ref":"eharbor_coordinator.html#pause/2","title":"eharbor_coordinator.pause/2","type":"function"},{"doc":"","ref":"eharbor_coordinator.html#start_link/1","title":"eharbor_coordinator.start_link/1","type":"function"},{"doc":"eharbor top level supervisor.","ref":"eharbor_sup.html","title":"eharbor_sup","type":"module"},{"doc":"","ref":"eharbor_sup.html#init/1","title":"eharbor_sup.init/1","type":"function"},{"doc":"","ref":"eharbor_sup.html#start_link/0","title":"eharbor_sup.start_link/0","type":"function"},{"doc":"eharbor is an innovative Erlang library designed to enhance the resilience and efficiency of distributed systems under heavy load. Unlike traditional approaches that rely solely on queues, eharbor intelligently combines load-shedding, request deduplication, and concurrency control to provide a more robust solution to system overload.","ref":"readme.html","title":"eharbor","type":"extras"},{"doc":"Intelligent Load-Shedding : Goes beyond simple queuing to proactively manage system load, preventing overload before it occurs. Request Deduplication : Efficiently handles multiple identical requests by processing a single instance and sharing the result, significantly reducing redundant workload. Concurrency Management : Limits the number of concurrent operations, protecting backend systems (like databases) from being overwhelmed. Configurable Buffering : Offers fine-tuned control over buffer sizes and system limits, ensuring optimal performance tailored to specific operational needs.","ref":"readme.html#key-features","title":"eharbor - Key Features","type":"extras"},{"doc":"Traditional overload protection mechanisms often rely heavily on queuing requests. However, queues alone cannot prevent system overload; they merely postpone the problem. eharbor addresses this by implementing an intelligent system that manages queued requests, actively reduces unnecessary workload, and controls the concurrency of task processing.","ref":"readme.html#why-eharbor","title":"eharbor - Why eharbor?","type":"extras"},{"doc":"To install eharbor , add it to your rebar.config dependencies: { deps , [ { eharbor , &quot;1.0.0&quot; } ] } . Then run rebar3 to fetch and compile the library: $ rebar3 compile","ref":"readme.html#installation","title":"eharbor - Installation","type":"extras"},{"doc":"Configuration First, set up your eharbor configuration: %% Sample configuration Config = \#{ name =&gt; my_harbor , backlog =&gt; 500 , %% Buffer size piers =&gt; 50 , %% Concurrency limit breakwater_limit =&gt; 1000 , %% Max waiting processes dedup =&gt; true , %% Enable request deduplication ordered =&gt; true %% Maintain operation order } . Launching eharbor Start an instance of eharbor with your configuration or add it to your supervisor tree: { ok , _ Pid } = eharbor : start_link ( Config ) . Running Tasks Use eharbor to run tasks efficiently: %% Execute the task through `eharbor` Result = eharbor : run ( fun some_mod : slow_function / 1 , [ Param ] , Config ) .","ref":"readme.html#getting-started","title":"eharbor - Getting Started","type":"extras"},{"doc":"Basic Usage %% Start eharbor { ok , _ Pid } = eharbor : start_link ( Config ) . %% Run the function via eharbor Result = eharbor : run ( fun some_mod : slow_function / 1 , [ Arg ] , Config ) . Ignoring Parameters Some parameters sometimes need to be excluded from deduplication matching. %% Start eharbor { ok , _ Pid } = eharbor : start_link ( Config ) . %% Define a resource-intensive function SourceIP = . . . IntensiveFun = fun ( Arg ) -&gt; some_mod : heavy_computation ( Arg , SourceIP ) end . %% Run the function via eharbor Result = eharbor : run ( IntensiveFun , [ Arg ] , Config ) .","ref":"readme.html#examples","title":"eharbor - Examples","type":"extras"},{"doc":"For detailed documentation, including advanced configurations and operational guidelines, please refer to our repository's doc directory or visit our online documentation .","ref":"readme.html#documentation","title":"eharbor - Documentation","type":"extras"},{"doc":"Contributions to eharbor are welcome! Please feel free to contribute, whether it's feature requests, bug reports, or code contributions. For code contributions, please fork the repository and submit a pull request with your changes.","ref":"readme.html#contributing","title":"eharbor - Contributing","type":"extras"},{"doc":"eharbor is released under the Apache License 2.0. Please look at the LICENSE file for more details.","ref":"readme.html#license","title":"eharbor - License","type":"extras"},{"doc":"Apache License Version 2.0, January 2004 http://www.apache.org/licenses/ TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 1. Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. 2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. 3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. 4. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License; and (b) You must cause any modified files to carry prominent notices stating that You changed the files; and (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. 5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. 6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. 7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. 8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. 9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS Copyright 2022, Eric des Courtis &lt;eric.descourtis@fireworkhq.com&gt;. Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"license.html","title":"LICENSE","type":"extras"}]